@@ -54,7 +54,7 @@ static WERROR check_one_prerequisite(struct dns_server *dns,
 	bool found = false;
 	struct dnsp_DnssrvRpcRecord *rec = NULL;
 	struct dnsp_DnssrvRpcRecord *ans;
-	uint16_t acount;
+	uint16_t a_count;
 
 	size_t host_part_len = 0;
 
@@ -82,19 +82,19 @@ static WERROR check_one_prerequisite(struct dns_server *dns,
 		if (pr->rr_type == DNS_QTYPE_ALL) {
 			/*
 			 */
-			werror = dns_lookup_records(dns, mem_ctx, dn, &ans, &acount);
+			werror = dns_lookup_records(dns, mem_ctx, dn, &ans, &a_count);
 			if (W_ERROR_EQUAL(werror, WERR_DNS_ERROR_NAME_DOES_NOT_EXIST)) {
 				return DNS_ERR(NAME_ERROR);
 			}
 			W_ERROR_NOT_OK_RETURN(werror);
 
-			if (acount == 0) {
+			if (a_count == 0) {
 				return DNS_ERR(NAME_ERROR);
 			}
 		} else {
 			/*
 			 */
-			werror = dns_lookup_records(dns, mem_ctx, dn, &ans, &acount);
+			werror = dns_lookup_records(dns, mem_ctx, dn, &ans, &a_count);
 			if (W_ERROR_EQUAL(werror, WERR_DNS_ERROR_NAME_DOES_NOT_EXIST)) {
 				return DNS_ERR(NXRRSET);
 			}
@@ -103,7 +103,7 @@ static WERROR check_one_prerequisite(struct dns_server *dns,
 			}
 			W_ERROR_NOT_OK_RETURN(werror);
 
-			for (i = 0; i < acount; i++) {
+			for (i = 0; i < a_count; i++) {
 				if (ans[i].wType == (enum dns_record_type) pr->rr_type) {
 					found = true;
 					break;
@@ -130,14 +130,14 @@ static WERROR check_one_prerequisite(struct dns_server *dns,
 		if (pr->rr_type == DNS_QTYPE_ALL) {
 			/*
 			 */
-			werror = dns_lookup_records(dns, mem_ctx, dn, &ans, &acount);
+			werror = dns_lookup_records(dns, mem_ctx, dn, &ans, &a_count);
 			if (W_ERROR_EQUAL(werror, WERR_OK)) {
 				return DNS_ERR(YXDOMAIN);
 			}
 		} else {
 			/*
 			 */
-			werror = dns_lookup_records(dns, mem_ctx, dn, &ans, &acount);
+			werror = dns_lookup_records(dns, mem_ctx, dn, &ans, &a_count);
 			if (W_ERROR_EQUAL(werror, WERR_DNS_ERROR_NAME_DOES_NOT_EXIST)) {
 				werror = WERR_OK;
 			}
@@ -145,7 +145,7 @@ static WERROR check_one_prerequisite(struct dns_server *dns,
 				werror = WERR_OK;
 			}
 
-			for (i = 0; i < acount; i++) {
+			for (i = 0; i < a_count; i++) {
 				if (ans[i].wType == (enum dns_record_type) pr->rr_type) {
 					found = true;
 					break;
@@ -170,7 +170,7 @@ static WERROR check_one_prerequisite(struct dns_server *dns,
 
 	*final_result = false;
 
-	werror = dns_lookup_records(dns, mem_ctx, dn, &ans, &acount);
+	werror = dns_lookup_records(dns, mem_ctx, dn, &ans, &a_count);
 	if (W_ERROR_EQUAL(werror, WERR_DNS_ERROR_NAME_DOES_NOT_EXIST)) {
 		return DNS_ERR(NXRRSET);
 	}
@@ -182,10 +182,10 @@ static WERROR check_one_prerequisite(struct dns_server *dns,
 	rec = talloc_zero(mem_ctx, struct dnsp_DnssrvRpcRecord);
 	W_ERROR_HAVE_NO_MEMORY(rec);
 
-	werror = dns_rr_to_dnsp(rec, pr, rec, dns_name_is_static(ans, acount));
+	werror = dns_rr_to_dnsp(rec, pr, rec, dns_name_is_static(ans, a_count));
 	W_ERROR_NOT_OK_RETURN(werror);
 
-	for (i = 0; i < acount; i++) {
+	for (i = 0; i < a_count; i++) {
 		if (dns_record_match(rec, &ans[i])) {
 			found = true;
 			break;
@@ -394,8 +394,8 @@ static WERROR handle_one_update(struct dns_server *dns,
 	bool needs_add = false;
 	bool name_is_static;
 
-	DEBUG(2, ("Looking at record: \n"));
-	if (DEBUGLVL(2)) {
+	DBG_NOTICE("Looking at record: \n");
+	if (DEBUGLVL(DBGLVL_NOTICE)) {
 		NDR_PRINT_DEBUG(dns_res_rec, discard_const(update));
 	}
 
@@ -418,10 +418,12 @@ static WERROR handle_one_update(struct dns_server *dns,
 	}
 
 	werror = dns_name2dn(dns, mem_ctx, update->name, &dn);
+	DBG_DEBUG("dns_name2dn(): %s\n", win_errstr(werror));
 	W_ERROR_NOT_OK_RETURN(werror);
 
 	werror = dns_common_lookup(dns->samdb, mem_ctx, dn,
 				   &recs, &rcount, &tombstoned);
+	DBG_DEBUG("dns_common_lookup(): %s\n", win_errstr(werror));
 	if (W_ERROR_EQUAL(werror, WERR_DNS_ERROR_NAME_DOES_NOT_EXIST)) {
 		needs_add = true;
 		werror = WERR_OK;
@@ -439,7 +441,7 @@ static WERROR handle_one_update(struct dns_server *dns,
 		if (rcount != 1) {
 			DBG_WARNING("Tombstoned dnsNode has %u records, "
 				    "expected 1\n", rcount);
-			if (DEBUGLVL(1)) {
+			if (DEBUGLVL(DBGLVL_WARNING)) {
 				NDR_PRINT_DEBUG(dns_res_rec, discard_const(update));
 			}
 		}
@@ -459,7 +461,6 @@ static WERROR handle_one_update(struct dns_server *dns,
 					DEBUG(5, ("Skipping update\n"));
 					return WERR_OK;
 				}
-				break;
 			}
 
 			/*
@@ -474,11 +475,13 @@ static WERROR handle_one_update(struct dns_server *dns,
 
 			werror = dns_rr_to_dnsp(
 			    recs, update, &recs[rcount], name_is_static);
+			DBG_DEBUG("dns_rr_to_dnsp(CNAME): %s\n", win_errstr(werror));
 			W_ERROR_NOT_OK_RETURN(werror);
 			rcount += 1;
 
 			werror = dns_replace_records(dns, mem_ctx, dn,
 						     needs_add, recs, rcount);
+			DBG_DEBUG("dns_replace_records(CNAME): %s\n", win_errstr(werror));
 			W_ERROR_NOT_OK_RETURN(werror);
 
 			return WERR_OK;
@@ -527,6 +530,7 @@ static WERROR handle_one_update(struct dns_server *dns,
 
 			werror = dns_rr_to_dnsp(
 			    mem_ctx, update, &recs[i], name_is_static);
+			DBG_DEBUG("dns_rr_to_dnsp(SOA): %s\n", win_errstr(werror));
 			W_ERROR_NOT_OK_RETURN(werror);
 
 			/*
@@ -539,7 +543,7 @@ static WERROR handle_one_update(struct dns_server *dns,
 					continue;
 				}
 				DBG_ERR("Duplicate SOA records found.\n");
-				if (DEBUGLVL(0)) {
+				if (DEBUGLVL(DBGLVL_ERR)) {
 					NDR_PRINT_DEBUG(dns_res_rec,
 							discard_const(update));
 				}
@@ -550,6 +554,7 @@ static WERROR handle_one_update(struct dns_server *dns,
 
 			werror = dns_replace_records(dns, mem_ctx, dn,
 						     needs_add, recs, rcount);
+			DBG_DEBUG("dns_replace_records(SOA): %s\n", win_errstr(werror));
 			W_ERROR_NOT_OK_RETURN(werror);
 
 			return WERR_OK;
@@ -561,9 +566,12 @@ static WERROR handle_one_update(struct dns_server *dns,
 
 		werror =
 		    dns_rr_to_dnsp(recs, update, &recs[rcount], name_is_static);
+		DBG_DEBUG("dns_rr_to_dnsp(GENERIC): %s\n", win_errstr(werror));
 		W_ERROR_NOT_OK_RETURN(werror);
 
 		for (i = first; i < rcount; i++) {
+			struct dnsp_DnssrvRpcRecord orig_rec = recs[i];
+
 			if (!dns_record_match(&recs[i], &recs[rcount])) {
 				continue;
 			}
@@ -576,6 +584,16 @@ static WERROR handle_one_update(struct dns_server *dns,
 			recs[i].flags = 0;
 			werror = dns_replace_records(dns, mem_ctx, dn,
 						     needs_add, recs, rcount);
+			DBG_DEBUG("dns_replace_records(REPLACE): %s\n", win_errstr(werror));
+			if (W_ERROR_EQUAL(werror, WERR_ACCESS_DENIED) &&
+			    !needs_add &&
+			    orig_rec.dwTtlSeconds == recs[i].dwTtlSeconds)
+			{
+				DBG_NOTICE("dns_replace_records(REPLACE): %s "
+					    "=> skip no-op\n",
+					    win_errstr(werror));
+				werror = WERR_OK;
+			}
 			W_ERROR_NOT_OK_RETURN(werror);
 
 			return WERR_OK;
@@ -583,6 +601,7 @@ static WERROR handle_one_update(struct dns_server *dns,
 		/* we did not find a matching record. This is new. */
 		werror = dns_replace_records(dns, mem_ctx, dn,
 					     needs_add, recs, rcount+1);
+		DBG_DEBUG("dns_replace_records(ADD): %s\n", win_errstr(werror));
 		W_ERROR_NOT_OK_RETURN(werror);
 
 		return WERR_OK;
@@ -637,6 +656,7 @@ static WERROR handle_one_update(struct dns_server *dns,
 
 		werror = dns_replace_records(dns, mem_ctx, dn,
 					     needs_add, recs, rcount);
+		DBG_DEBUG("dns_replace_records(DELETE-ANY): %s\n", win_errstr(werror));
 		W_ERROR_NOT_OK_RETURN(werror);
 
 		return WERR_OK;
@@ -655,6 +675,7 @@ static WERROR handle_one_update(struct dns_server *dns,
 
 			werror = dns_rr_to_dnsp(
 			    ns_rec, update, ns_rec, name_is_static);
+			DBG_DEBUG("dns_rr_to_dnsp(NS): %s\n", win_errstr(werror));
 			W_ERROR_NOT_OK_RETURN(werror);
 
 			for (i = first; i < rcount; i++) {
@@ -673,6 +694,7 @@ static WERROR handle_one_update(struct dns_server *dns,
 
 		werror =
 		    dns_rr_to_dnsp(del_rec, update, del_rec, name_is_static);
+		DBG_DEBUG("dns_rr_to_dnsp(DELETE-NONE): %s\n", win_errstr(werror));
 		W_ERROR_NOT_OK_RETURN(werror);
 
 		for (i = first; i < rcount; i++) {
@@ -685,6 +707,7 @@ static WERROR handle_one_update(struct dns_server *dns,
 
 		werror = dns_replace_records(dns, mem_ctx, dn,
 					     needs_add, recs, rcount);
+		DBG_DEBUG("dns_replace_records(DELETE-NONE): %s\n", win_errstr(werror));
 		W_ERROR_NOT_OK_RETURN(werror);
 	}
 
@@ -717,6 +740,7 @@ static WERROR handle_updates(struct dns_server *dns,
 	}
 
 	werror = dns_name2dn(dns, tmp_ctx, zone->name, &zone_dn);
+	DBG_DEBUG("dns_name2dn(): %s\n", win_errstr(werror));
 	W_ERROR_NOT_OK_GOTO(werror, failed);
 
 	ret = ldb_transaction_start(dns->samdb);
@@ -733,23 +757,20 @@ static WERROR handle_updates(struct dns_server *dns,
 	for (ri = 0; ri < upd_count; ri++) {
 		werror = handle_one_update(dns, tmp_ctx, zone,
 					   &updates[ri], tkey);
+		DBG_DEBUG("handle_one_update(%u): %s\n",
+			  ri, win_errstr(werror));
 		W_ERROR_NOT_OK_GOTO(werror, failed);
 	}
 
-	ldb_transaction_commit(dns->samdb);
-	TALLOC_FREE(tmp_ctx);
-
-	if (tkey != NULL) {
-		ldb_set_opaque(
-			dns->samdb,
-			DSDB_SESSION_INFO,
-			system_session(dns->task->lp_ctx));
-	}
-
-	return WERR_OK;
-
 failed:
-	ldb_transaction_cancel(dns->samdb);
+	if (W_ERROR_IS_OK(werror)) {
+		ret = ldb_transaction_commit(dns->samdb);
+		if (ret != LDB_SUCCESS) {
+			werror = DNS_ERR(SERVER_FAILURE);
+		}
+	} else {
+		ldb_transaction_cancel(dns->samdb);
+	}
 
 	if (tkey != NULL) {
 		ldb_set_opaque(
@@ -857,10 +878,12 @@ WERROR dns_server_process_update(struct dns_server *dns,
 	*update_count = in->nscount;
 	*updates = in->nsrecs;
 	werror = update_prescan(in->questions, *updates, *update_count);
+	DBG_DEBUG("update_prescan(): %s\n", win_errstr(werror));
 	W_ERROR_NOT_OK_RETURN(werror);
 
 	werror = handle_updates(dns, mem_ctx, in->questions, *prereqs,
 			        *prereq_count, *updates, *update_count, tkey);
+	DBG_DEBUG("handle_updates(): %s\n", win_errstr(werror));
 	W_ERROR_NOT_OK_RETURN(werror);
 
 	return werror;
